#' calcFFT
#'
#' This function calculates FFT on read data
#' @param d Dataframe got from infrasonic.readSerial()
#' @param res Frequency resolution. Must fit to the data length. For lower resolution a sliding window is calculated and averaged. Defaults to 0.1
#' @param noise Sensor noise in Pa to substract from signal
#' @param t_abs_lim Select a subset of the time series. Defaults to null
#' @param matrix_output return a matrix instead of averaged FFT
#' @param mean Mean calculation by squared or linear pressure. Defaults to squared
#' @param detrend Detrend the signal to avoid edge effects
#' @keywords FFT
#' @name infrasonic.calcFFT
#' @export
#' @examples
#' infrasonic.calcFFT(d,res=0.01)

infrasonic.calcFFT<-function(d,res=0.1,noise=0,t_abs_lim=NULL,
                             matrix_output=FALSE,
                             detrend=FALSE,mean='squared'){
  if(! is.null(t_abs_lim)){
    if(class(t_abs_lim)=='character'){
      t_abs_lim=as.POSIXct(t_abs_lim)
    }
    d=subset(d,t_abs>=t_abs_lim[1] & t_abs<=t_abs_lim[2])
    d$t=d$t-as.numeric(d$t[1])
  }
  t=d[,1] # time vector
  s=d[,2] # signal vector
  n=length(t) # number of data points
  r=(n-1)/(max(t)-min(t)) #resolution of series in Hz
  ## Window length for res Spectra
  w=round(r/res)
  if(w %% 2 ==0) w=w-1
  i=1:((w-1)/2)
  freq=(i-1)*r/w # frequency vector for FFT

  if(w>n){
    warning("Sample is to short")
    return(NULL)
  }

  #calculate matrix off ffts in abs values
  step=round(w/2) ## move 1/2
  m=sapply(0:round((n-w)/step), function(x){
    t_w=t[(x*step+1):(x*step+w)]
    s_w=s[(x*step+1):(x*step+w)]
    if(detrend){
      slope=(s[x*step+w]-s[x*step+1])/(t[x*step+w]-t[x*step+1])
      intercept=s[x*step+1]-t[x*step+1]*slope
      abs(fft(s_w-(slope*t_w+intercept)))
    } else {
      abs(fft(s_w))
    }
  })
  #calculate vector linear or squared average of ffts
  if(mean=='squared') ff_mean=sqrt(rowMeans(m^2,na.rm=TRUE))
  else ff_mean=rowMeans(m,na.rm=TRUE)
  #calculate a noise level. This might go more easy ;-)
  #noise might be usefull when comparing different sensors
  #but does not work perfect. Noise might be dropped in future
  if(noise){
    n=rnorm(length(t))*noise
    n=infrasonic.calcFFT(data.frame(t,n),res=res)
    noise=mean(20e-6*10^(n[,2]/20))
  }
  norm_ref=20e-6 ## 20 µPa
  if(matrix_output){
    db=20*log10((2*m[i,]/w-noise)/sqrt(2)/norm_ref)
    return(list(db=db[-1,],
            freq=freq[-1],
           t=t[(0:round((n-w)/step))*step+1]))
  }
  #calculate one sided fft
  ff=(2*ff_mean[i]/w-noise)
  ff[ff<1e-6]=1e-6 # this is done to avoid negative values when working with noise
  db=20*log10(ff/sqrt(2)/norm_ref)
  data.frame(freq=freq[-1],db=db[-1])
}


#' calc_third_bands
#'
#' This function calculates the 1/3 octave bands for given infrasonic
#' frequencies. The function returns a dataFrame with frequency,
#' db-band value and perception threshold
#' @param d Dataframe got from infrasonic.readSerial()
#' @param res Frequency resolution. Must fit to the data length. For lower resolution a sliding window is calculated and averaged. Defaults to 0.1
#' @param noise Sensor noise in Pa to substract from signal
#' @param t_abs_lim Select a subset of the time series. Defaults to null
#' @keywords calculate thirds
#' @name infrasonic.calcThirds
#' @export
#' @examples
#' infrasonic.calcThirds(d,res=0.02)
infrasonic.calcThirds<-function(d,res=0.1,noise=0,t_abs_lim=NULL){
  fft=infrasonic.calcFFT(d,res,noise,t_abs_lim)
  freq=fft$freq
  ff=20e-6*10^(fft$db/20)
  ## Resolution Terz
  ## Wahrnehmungsschelle aus Tabelle A3-1 LUBW tieffrequente Geräusche inkl Infraschall
  hs_f=c(0.992,1.25,1.575,1.984,2.5,3.15,3.969,5,6.300,7.937,10,12.5599,15.874,20)
  hs_db=c(NA,NA, 124, 122, 120,117,113,108.5,105,100,92,84,76,69)
  ff_t=numeric()
  freq_t=hs_f
  count=numeric()
  lower_limit=numeric()
  upper_limit=numeric()
  for(j in 1:length(freq_t)){
    lower_limit[j]=freq_t[j]/2^(1/6)
    upper_limit[j]=freq_t[j]*2^(1/6)
    count[j]=sum(freq>lower_limit[j] & freq<upper_limit[j])
    ff_t[j]=sum(ff[freq>lower_limit[j] & freq<upper_limit[j]]^2);
  }
  db_t=10*log10(ff_t/((20e-6)^2))
  data.frame(freq=hs_f,db=db_t,ws=hs_db,counts=count,freq_lower=lower_limit,freq_upper=upper_limit)
}


#' calc_signal
#'
#' This function calculates the signal in a defined frequency range above
#' a baseline range. It returns the signal in dB
#' @param ff Dataframe got from infrasonic.calcFFT()
#' @param peak Frequency limits of peak
#' @param base Frequency limit of base
#' @param plot make a plot
#' @keywords calculate signal
#' @name infrasonic.calcSignal
#' @export
#' @examples
#' infrasonic.calcSignal(ff,c(2.1,2.32),c(1.9,2.45),T))
infrasonic.calcSignal<-function(ff,peak,base,plot=FALSE){
  s_peak=ff$freq>=peak[1] & ff$freq<=peak[2]
  s=ff$freq>=base[1] & ff$freq<=base[2]
  s_base=ff$freq>=base[1] & ff$freq<=base[2] & (! s_peak)
  p2=(10^(ff$db/20))^2
  x=ff$freq[s_base]
  y=ff$db[s_base]
  m=lm(y~x)
  db_base=predict(m,data.frame(x=ff$freq))
  if(plot){
    plot(ff$freq[s],p2[s],xlab='Frequenz [Hz]',ylab='P^2 re 20µPa',type='l')
    abline(v=min(ff$freq[s_peak]),col=2)
    abline(v=max(ff$freq[s_peak]),col=2)
    lines(ff$freq[s],(10^(db_base[s]/20))^2,col=3)
  }
  10*log10(sum(p2[s_peak])-sum((10^(db_base[s_peak]/20))^2))
}
