#' saveData
#'
#' This function saves infrasonic data with a prefix and start timestamp automatically appended
#' @param d Dataframe got from infrasonic.readSerial()
#' @param prefix Prefix for filename for saving. Datetime is automatically appended
#' @keywords save infrasonic data
#' @name infrasonic.saveData
#' @export
#' @examples
#' infrasonic.saveData(d,"SDP610-WEA200m")
infrasonic.saveData<-function(d,prefix){
  f=paste(prefix,d[1,3],'csv',sep=".")
  d$t_abs[2:length(d$t_abs)]=NA
  write.csv2(d,f,row.names = FALSE)
}

#' readRaw
#'
#' This function reads infrasonic data from a Serial2SD logger
#' @param file filename to read
#' @param freq Measureing frequency
#' @keywords read raw infrasonic data
#' @name infrasonic.readRaw
#' @export
#' @examples
#' infrasonic.readRaw("ku-0413b.csv",50)
infrasonic.readRaw<-function(file,freq=128/3){
  d=read.table(file,nrows=1,comment.char = '/')
  ts=as.POSIXct(paste(as.character(d$V2[1]),as.character(d$V3[1])),"GMT")
  ts=as.POSIXct(as.numeric(ts),origin = '1970-01-01') # local timezone
  d=read.csv(file,header=TRUE,fileEncoding = 'iso8859-1')
  v=d[,1]
  if(class(v)!="numeric"){
    v=as.numeric(as.character(v))
    v=v[! is.na(v)]
  }
  t=(1:length(v))/freq
  t_abs=ts+t
  data.frame(t,v,t_abs)
}

#' cat files
#'
#' This function concatinates infrasonic data
#' @param ... dateframes or files to concatinate
#' @keywords concatinate infrasonic data
#' @name infrasonic.c
#' @export
#' @examples
#' infrasonic.c(d1,d2,d3)
infrasonic.c<-function(...){
  l=list(...)
  if(class(l[[1]])=="character"){
    f=l
    for(i in 1:length(l)){
      print(paste("opening",l[[i]]))
      l[[i]]=infrasonic.readRaw(l[[i]])
    }
  }
  v=l[[1]]$v
  e=max(l[[1]]$t_abs)
  for(i in 2:length(l)){
    s=min(l[[i]]$t_abs)
    gap=round((as.numeric(s)-as.numeric(e))*128/3)
    if(gap<1) gap=1
    v=c(v,rep(NA,gap),l[[i]]$v)
    e=max(l[[i]]$t_abs)
  }
  s=l[[1]]$t_abs[1]
  t=((1:length(v))-1)*3/128
  t_abs=s+t
  data.frame(t,v,t_abs)

}


#' generate time series
#'
#' This function generates time series with a given sinus signal on top of white noise or a measures series
#' @param signal signal strenth in dB
#' @param noise amplitude of noise in dB or a data.frame with measured signal
#' @param duration duration of sample
#' @param m_freq measurement frequency
#' @param s_freq frequency of signal
#' @param mod_amp amplitude of frequency modulation
#' @param mod_freq frequency of frequency modulation
#' @keywords generate infrasonic time series
#' @name infrasonic.generateSignal
#' @export
#' @examples
#' d=infrasonic.generateSignal()
#' infrasonic.plotFFT(d)
infrasonic.generateSignal<-function(signal=60,noise=60,
                                    duration=600,m_freq=100,
                                    s_freq=2,
                                    mod_amp=0,mod_freq=1/300){
  p_signal=2e-5*10^(signal/20)
  if(class(noise)=='data.frame'){
    t=noise[,1]
    tt=t+mod_amp/mod_freq/2/pi*sin(mod_freq*2*pi*t)
    v=noise[,2]+p_signal*sin(tt*2*pi*s_freq)
  } else {
    p_noise=2e-5*10^(noise/20)
    t=seq(0,duration,1/m_freq)
    tt=t+mod_amp/mod_freq/2/pi*sin(mod_freq*2*pi*t)
    v=p_noise*rnorm(length(t))+p_signal*sin(tt*2*pi*s_freq)
  }
  data.frame(t,v)
}
#' compareFiles
#'
#' This function read incoming serial data
#' it plots the data online and returns da data frame with the collected data
#' @param freq_lim frequency limits to plot. Defaults to c(0,20)
#' @param res Frequency resolution. Must fit to the data length. For lower resolution a sliding window is calculated and averaged. Defaults to 0.1
#' @param files Array of filenames to read for comparing
#' @param names Corresponding names for files. If set these names are used for the legend instead of the file names
#' @param db_lim db limits in Plot. Defaults to c(0,20)
#' @param noise Pa values of noise to substract
#' @param t_lim limits for time series plotting. If zero no time series plot is made. Defaults to c(0,0)
#' @param main Title of plot
#' @keywords compare FFT readings
#' @name infrasonic.compareFiles
#' @export
#' @examples
#' infrasonic.compareFiles(c("SDP600-Harsdorf-Fenster-Auf.2020-03-18 22:27:03.csv",
#'   "SDP600-Harsdorf-Fenster-Zu.2020-03-18 22:15:23.csv",
#'   "SDP600-Harsdorf-3600s.2020-03-18 23:05:47.csv"),
#'   c("Fenster auf","Fenster zu","Fenster zu (1h)"),
#'   res=0.03,max_f=25)

infrasonic.compareFiles<-function(files,names=NULL,res=0.1,
                                  freq_lim=c(0,20),
                                  db_lim=c(20,120),
                                  t_lim=c(0,0),main='',
                                  noise=NULL){
  if(t_lim[1]){
    i=1
    par(mfrow=c(1,2))
    for(f in files){
      d=read.csv2(f)
      selected=d$t<=t_lim[1] & d$t>=t_lim[0]
      v_zero=mean(d$v[selected])
      if(length(files)==1) v_zero=0
      if(i==1) plot(d$t[selected],d$v[selected]-v_zero,xlim=t_lim,xlab='Zeit [s]',ylab='Druck [Pa]',type='l')
      else lines(d$t[selected],d$v[selected]-v_zero,col=i)
      i=i+1
    }
  } else par(mfrow=c(1,1))

  i=1
  for(f in files){
    pa_noise=0
    d=read.csv2(f)
    if(length(noise)){
      if(length(noise)>1) pa_noise=noise[i]
      else pa_noise=noise
    }
    d=infrasonic.calcFFT(d,res=res,noise=pa_noise)
    if(i==1) plot(d[,1],d[,2],xlim=freq_lim,ylim=db_lim,
                  xlab='Frequenz[Hz]',
                  ylab=paste('Lautstärke [dB 20µPa] (Pegelbreite',res,'Hz)'),main=main,type='l')
    else lines(d[,1],d[,2],col=i)
    i=i+1
  }
  if(is.null(names)) names=files
  legend('topright',names,col=1:length(names),lty=1)

}


