#' plotFFT
#'
#' This function calculates and plots FFT on read data
#' when x_lim[1] is greater than zero, also the time series is plotted for the selected
#' time interval
#' @param d Dataframe got from infrasonic.readSerial() or from infrasonic.readRaw()
#' @param res Frequency resolution. Must fit to the data length. For lower resolution a sliding window is calculated and averaged. Defaults to 0.1
#' @param freq_lim frequency limit for plot. Defaults to c(0,20)
#' @param db_lim db limits on y-axis. Defaults to c(20,120)
#' @param t_lim Limits for time series plotting. If zero no time series plot is made. Defaults to c(0,0)
#' @param t_abs_lim Select a subset of the time series. Defaults to null
#' @param noise Sensor noise in Pa to substract from signal
#' @param plot_third add thirds to plot
#' @param plot_threshold add threshold to plot
#' @param main Title of plot
#' @name infrasonic.plotFFT
#' @keywords plot FFT
#' @export
#' @examples
#' infrasonic.plotFFT(d,t_lim=(10,20)) # Plot FFT and the time series
infrasonic.plotFFT<-function(d,res=0.1,freq_lim=c(0,20),
                             db_lim=c(20,120),main='',
                             t_lim=c(0,0),t_abs_lim=NULL,noise=0,
                             plot_third=TRUE,plot_threshold=FALSE,...){
  if(! is.null(t_abs_lim)){
    if(class(t_abs_lim)=='character'){
      t_abs_lim=as.POSIXct(t_abs_lim)
    }
    d=subset(d,t_abs>=t_abs_lim[1] & t_abs<=t_abs_lim[2])
    d$t=d$t-as.numeric(d$t[1])
  }
  t=d[,1]
  s=d[,2]
  if(t_lim[2]>0){
    par(mfrow=c(1,2))
    selected=t<=t_lim[2] & t>=t_lim[1]
    plot(t[selected],s[selected],xlim=t_lim,xlab='Zeit [s]',
         ylab='Druck [Pa]',type='l')
  }

  fft=infrasonic.calcFFT(d,res,noise)
  plot(fft$freq,fft$db,ylim=db_lim,type='l',main=main,
       xlim=freq_lim,xlab='Frequenz [Hz]',
       ylab='Lautstärke [dB] re 20µPa',...)

  if(plot_third){
    terz=infrasonic.calcThirds(d,res,noise)
    for(i in 1:length(terz$db)){
      lines(c(terz$freq_lower[i],terz$freq_lower[i],terz$freq_upper[i],terz$freq_upper[i]),
          c(0,terz$db[i],terz$db[i],0),col=2,lty=3)
    }
    l=c(paste(res,'Hz'),'Terz')
    lty=c(1,3)
    col=1:2
    if(plot_threshold) {
      lines(terz$freq,terz$ws,lty=2,col=3)
      l=c(l,'WS')
      lty=c(lty,2)
      col=c(col,3)
    }
    legend('topright',l,lty=c(1,3,2),col=1:3)
  } else {
    legend('topright',paste(res,'Hz'),lty=1,col=1)
  }
}

#' plotSpectrogram
#'
#' This function calculates and plots spectrogram on read data
#' @param d Dataframe got from infrasonic.readSerial() or from infrasonic.readRaw()
#' @param res Frequency resolution. Must fit to the data length. For lower resolution a sliding window is calculated and averaged. Defaults to 0.1
#' @param freq_lim frequency limit for plot. Defaults to c(0,20)
#' @param db_lim db limits on y-axis. Defaults to c(20,60)
#' @param xlim Limits for time axis
#' @param noise Sensor noise in Pa to substract from signal
#' @name infrasonic.plotSpectrogram
#' @keywords plot FFT
#' @export
#' @examples
#' infrasonic.plotSpectrogram(d,db_lim=c(25,50))
infrasonic.plotSpectrogram<-function(d,res=0.1,freq_lim=c(0,20),
                             db_lim=c(20,60),main='',
                             xlim=NULL,noise=0,...){
  if(! is.null(xlim)){
    if(class(xlim)=='character'){
      xlim=as.POSIXct(xlim)
    }
    d=subset(d,t_abs>=xlim[1] & t_abs<=xlim[2])
    d$t=d$t-as.numeric(d$t[1])
  }
  x=infrasonic.calcFFT(d,matrix_output = TRUE,res)
  imagep(x = as.POSIXct(as.character(d$t_abs[1]))+x$t,
       y = x$freq,
       z = t(x$db),
       ylim = freq_lim,
       zlim = db_lim,
       #    col = oce.colorsViridis,
       col = oce.colorsTemperature,
       ylab = paste('Frequenz [Hz] (Pegelbreite',res,'Hz)'),
       xlab = 'Time [s]',
       zlab = 'dB re 20µPa',
       drawPalette = T,
       ...)
}

