#' readSerial
#'
#' This function read incoming serial data
#' it plots the data online and returns da data frame with the collected data
#' @param runtime Time in secondsto collect data. Defaults to 60 seconds
#' @param ylim Limits for y axis
#' @param zero Collect a offset level at startup and substract from incoming values. Defaults to FALSE
#' @param signal2pa Signal to Pa-Factor. Defaults to 1
#' @param port Serial port to read from. Defaults to ttyUSB0
#' @param main Main title for plot window
#' @param send Text string to send to serial device at startup. Can be used to switch serial device
#' @param plot Live plot incoming data. Should be set to FALSE for long reading times. Defaults to TRUE.
#' @keywords serial read data
#' @export
#' @examples
#' d=infrasonic.readSerial(runtime=120,port='com3')

infrasonic.readSerial <- function(runtime = 60,
                             ylim = NULL,
                             zero = FALSE,
                             signal2pa = 1,
                             port="ttyUSB0",
                             main = '',
                             send = NULL,
                             plot = TRUE) {
  con <- serialConnection(name = port,
                          port = port,
                          mode = "38400,n,8,1")
  par(mfrow=c(1,1))
  close(con)
  open(con)
  Sys.sleep(0.1)
  read.serialConnection(con)
  stopTime <- Sys.time() + runtime + 2
  i = 0
  v = rep(NA, runtime * 100)
  v_zero=0
  dstart = 0
  freq = 0

  while (Sys.time() < stopTime)
  {
    newText <- read.serialConnection(con)
    if (!is.null(newText) ) {
      if(! is.null(send)){
        write.serialConnection(con,send)
        send=NULL
      }
      tmp = unlist(strsplit(newText, ' '))
      if (length(tmp) > 0) {
        tmp = as.numeric(tmp)
        tmp = tmp[!is.na(tmp)]*signal2pa
        if (length(tmp)) {
          ## Valid Data
          j = 1:length(tmp)
          if (i == 0) {
            dstart = Sys.time()
            print(dstart)
            stopTime=Sys.time()+runtime
            istart = length(tmp)
          }
          v[j + i] = tmp-v_zero
          i = i + length(tmp)
          if (!freq &&
              as.numeric(Sys.time() - dstart) > 1 && (i - istart) > 30) {
            freq = (i - istart) / as.numeric(Sys.time() - dstart)
            t = seq(0, 1.1*runtime+5, 1 / freq)
            if (length(ylim) == 1) {
              if(zero){
                v_zero=mean(v,na.rm=TRUE)
                v=v-v_zero
              }
              prez=0
              if(ylim<2) prez=1
              ylim = round(c(-1, 1) / 2 * ylim + mean(v,na.rm=TRUE),prez)
            }
            if(plot) plot(
              t[1:i],
              v[1:i],
              xlim = c(0, runtime),
              type = 'l',
              ylim = ylim,
              ylab = 'Druck [Pa]',
              xlab = 'Zeit [s]'
            )
          } else if (freq) {
            index = c(0, j) + (i - length(tmp))
            if(plot) lines(t[index], v[index], type = 'l')
          }
        }
      }

    }
    Sys.sleep(0.3)
  }
  d_end=Sys.time()
  print(d_end)
  v = v[!is.na(v)]
  t = (1:length(v))/length(v)*(as.numeric(d_end)-as.numeric(dstart))
  t_abs=dstart+t
  data.frame(t, v,t_abs)
}

calc.fft<-function(d,res=0.1){
  t=d[,1]
  s=d[,2]
  n=length(t)
  r=(n-1)/(max(t)-min(t))
  ## Window length for res Spectra
  w=round(r/res)
  if(w %% 2 ==0) w=w-1
  i=1:((w-1)/2)
  freq=(i-1)*r/w

  if(w>n/2){
    warning("Sample is to short")
    return(NULL)
  }

  step=round(w/3) ## move 1 third
  ff_mean=rowMeans(sapply(0:round((n-w)/step), function(x){
    abs(fft(s[(x*step+1):(x*step+w)]))
  }),na.rm=TRUE)
  ff=2*ff_mean[i]/w
  norm_ref=20e-6 ## 20 µPa
  db=20*log10(ff/norm_ref)
  data.frame(freq[-1],db[-1])
}

cat_function <- function(love=TRUE){
  if(love==TRUE){
    print("I love cats!")
  }
  else {
    print("I am not a cool person.")
  }
}
